import styled, { css } from 'styled-components';
import { colors, rem } from '../../utils';

/* eslint-disable-next-line */
export type MarginProps = {
  marginTop?:number , 
  marginBottom?:number , 
  marginLeft?:number , 
  marginRight?:number
}
export interface DividerProps {
  /** size predefines thickness of the divider
   * xx-small: 1px;
   * x-small: 2px;
   * small: 3px;
   * normal: 4px;
   * medium: 5px;
   * large: 6px;
   * x-large: 8px;
   * xx-large: 10px;
   * 
   * default size:small;
   */
  size ?: 'xx-small' |'x-small'| 'small'|'normal' | 'medium' | 'large' |'x-large'|'xx-large' |undefined,
  /** margin on divider
   * {
   *  marginTop: number ,
   *  marginBottom: number ,
   *  marginLeft: number ,
   *  marginRight: number
   * }
   * values are considered in px.
   */
  margin?:  MarginProps;

  /** color of divider 
   * default colors.gray100 // #EFEFEF
  */
  className?: string;
  id?: string;
  /**
   * color of divider
   */
  color?: string;
  /**
   * thickness of divider
   * overrides size property for thickness
   */
  thickness?:number;
  /**
   * type of divider
   * default its horizontal divider
   */
  type?: 'horizontal' | 'vertical';
  /**
   * style type of divider
   */
  styleType?: "dotted"|"dashed"|"solid"|"double"|undefined;
  /**
   * style for divider
   */
  style?: React.CSSProperties|any;
}

const dividerSizeMap = {
  "xx-small":{
    thickness:1
  },
  "x-small":{
    thickness:2
  },
  "small":{
    thickness:3
  },
  "normal":{
    thickness:4
  },
  "medium":{
    thickness:5
  },
  "large":{
    thickness:6
  },
  "x-large":{
    thickness:8
  },
  "xx-large":{
    thickness:10
  },
}

const StyledDividerVertical = styled.div<DividerProps>`
  ${({ margin }) => css`${margin}`}
  width: 0;
  height: inherit;
  display: inline-block;
  align-self:stretch;
  ${({ size = "small", color,thickness }) =>css`
      border-right: ${rem(thickness || dividerSizeMap[size]?.thickness)} solid ${color};
    `
  }
`

const StyledDivider = styled.div<DividerProps>`
  height:0!important;
  border-top:none!important;
  border-left:none!important;
  border-right:none!important;
  ${({ size = "small", color,thickness }) =>css`
      border-bottom: ${rem(thickness || dividerSizeMap[size]?.thickness)} solid ${color};
    `
  }
  ${({ margin }) => css`${margin}`}
  ${
    ({styleType})=>styleType && css`
      border-style:${styleType};`
  }
`;

export const Divider = ({
  size = 'small',
  margin ={},
  thickness,
  type="horizontal",
  color=colors.gray100,
  styleType='solid',
  ...props
}: DividerProps) => {
  if(type === 'horizontal') {
    return(
      <StyledDivider
        {...props}
        margin={margin}
        size={size}
        color={color}
        thickness={thickness}
       styleType={styleType}
      />
    )
  }
  return (
    <StyledDividerVertical
      {...props}
      size={size}
      margin={margin}
      color={color}
      thickness={thickness}
      styleType={styleType}
    >&nbsp;</StyledDividerVertical>
  );
};

export default Divider;
