import { useRef, useState, ReactNode, useEffect, CSSProperties } from 'react';
import styled from 'styled-components';
import { Popup as KPopup, PopupProps } from '@progress/kendo-react-popup';
export interface DropdownProps extends PopupProps {
  label: string | ReactNode;
  children?: ReactNode;
  appendTo?: HTMLElement;
  callout?: boolean;
  openOnHover?: boolean;
  /**
   * containerProps props for dropdown label container element
   */
  containerProps?: {
    id?: string;
    className?: string;
    style?: CSSProperties;
  };
}
const StyledPopover = styled(KPopup)`
  .k-popover-body {
    background: inherit;
    z-index: 9;
    padding: 0;
  }
  .dropdown-content {
    padding: 15px;
    &:focus {
      outline: none;
    }
  }
  .popup-callout {
    position: absolute;
    top: -5px;
    transform: rotate(45deg);
    background: inherit;
    width: 10px;
    height: 10px;
    border-left: 1px solid #ccc;
    border-top: 1px solid #ccc;
  }
  .popover-content {
    .k-popover-body {
      background: inherit;
    }
  }
`;
function getParentShadowDom(node: Node | null): {
  hasShadowDom?: boolean;
  shadowRoot?: Node;
} {
  let parent = node && node.parentNode;
  while (parent) {
    if (parent.toString() === '[object ShadowRoot]') {
      return { hasShadowDom: true, shadowRoot: parent };
    }
    parent = parent.parentNode;
  }
  return { hasShadowDom: false };
}
export const Dropdown = (props: DropdownProps) => {
  const anchor = useRef<HTMLLabelElement>(null);
  const [show, setShow] = useState<boolean>(false);
  const [clicked, setClicked] = useState<boolean>(false);
  const [parentShadowDom, setParentShadowDom] = useState<{
    hasShadowDom?: boolean;
    shadowRoot?: Node;
  }>({});
  const contentRef = useRef<HTMLDivElement>(null);
  const blurTimeoutRef = useRef<ReturnType<typeof setInterval> | null>(null);
  const [position, setPosition] = useState<{ left?: number; top?: number }>({
    left: 0,
    top: 0,
  });
  const {
    children,
    label,
    appendTo,
    callout,
    openOnHover = false,
    containerProps = {},
    ...rest
  } = props;
  useEffect(() => {
    setParentShadowDom(getParentShadowDom(anchor.current));
  }, []);
  useEffect(() => {
    if (show) {
      getPosition(anchor.current, contentRef.current);
    }
  }, [show]);
  const onOpen = () => {
    contentRef.current && contentRef.current.focus();
  };

  const onFocus = () => {
    // the user is still inside the content
    blurTimeoutRef.current && clearTimeout(blurTimeoutRef.current);
  };

  const onBlurTimeout = () => {
    // the user is now outside the popup
    setShow(false);
  };

  const onBlur = () => {
    blurTimeoutRef.current && clearTimeout(blurTimeoutRef.current);
    blurTimeoutRef.current = setTimeout(onBlurTimeout, 400);
  };

  function getPosition(anchor, content) {
    const anchorRect = anchor.getBoundingClientRect();
    const contentRect = content.getBoundingClientRect();
    const left = anchorRect.x - contentRect.x - 5 + anchorRect.width / 2;
    const top = anchorRect.y - contentRect.y - 5 + anchorRect.height / 2;
    setPosition({ left: left, top: top });
  }

  const onClick = () => {
    setShow((show) => {
      setClicked(!show);
      return !show;
    });
  };
  const onMouseOver = () => {
    openOnHover && setShow(true);
  };
  const onMouseLeave = () => {
    openOnHover && !clicked && setShow(false);
  };
  return (
    <div {...containerProps}>
      <label
        onClick={onClick}
        onMouseOver={onMouseOver}
        onMouseLeave={onMouseLeave}
        ref={anchor}
      >
        {label}
      </label>
      <StyledPopover
        anchor={anchor.current}
        show={show}
        onOpen={onOpen}
        //Doc: https://www.telerik.com/kendo-react-ui/components/popup/aligning-positioning/
        collision={{
          vertical: 'flip',
          horizontal: 'fit',
        }}
        anchorAlign={{
          vertical: 'bottom',
          horizontal: 'center',
        }}
        margin={{
          vertical: 10,
          horizontal: 10,
        }}
        popupAlign={{
          vertical: 'top',
          horizontal: 'center',
        }}
        appendTo={
          appendTo || (parentShadowDom.shadowRoot?.firstChild as HTMLElement)
        }
        style={{ width: 250 }}
        {...rest}
      >
        <div
          ref={contentRef}
          className={'dropdown-content'}
          tabIndex={0}
          onFocus={onFocus}
          onBlur={onBlur}
        >
          {children}
        </div>
        {callout && (
          <div className="popup-callout" style={{ left: position.left }}></div>
        )}
      </StyledPopover>
    </div>
  );
};

export default Dropdown;
