import styled, { css } from 'styled-components';
import {
  ListView as KListView,
  ListViewProps as KListViewProps,
  ListViewHeaderProps as KListViewHeaderProps,
  ListViewHeader as KListViewHeader,
  ListViewFooterProps as KListViewFooterProps,
  ListViewFooter as KListViewFooter,
  ListViewItemProps,
} from "@progress/kendo-react-listview";
import { ComponentType, useCallback } from 'react';
import Divider from '../divider/divider';
import { colors, rem } from '../../utils';
import Button from '../button/button';

/* eslint-disable-next-line */
export interface ListViewProps extends KListViewProps{
  // kendo ListView prop
  header?: ComponentType<KListViewHeaderProps>| undefined,
  // kendo ListView prop
  footer?: ComponentType<KListViewFooterProps> | undefined,
  // kendo ListView prop
  item?: ComponentType<ListViewItemProps>| undefined,
  /**
   * divider property
   * show/hide divider line after each item render
   */
  divider?:boolean;
  /**
   * direction property
   * determines the direction of list items
   */
  direction?: "vertical" | "horizontal";
  /**
   * disabled property
   * Allows to disable all list items
   */
  disabled?: boolean;
  /**
   * onClick property
   * onClick callback method
   * triggers when clicked on list item which is not disabled
   * callback arguments (dataItem,index,event)
   */
  onClick?:(dataItem?:object,index?:number,e?:object)=>void;
  /**
   * dataItemKey property
   * when passed data is array of object then dataItemKey is mendatory
   * to render list item dataItem[dataItemKey]
   */
  dataItemKey?:string;
  /**
   * as property
   * allows list item to be rendered as `button | a | span | div` element
   */
  as?: 'button' | 'a' | 'span' | 'div',
}

const StyledListView = styled(KListView)<ListViewProps>`
  .k-listview-content{
    display:flex;
    flex-grow:1;
    flex-direction:${({direction})=> direction==="vertical"?'column':"row"};
    ${({divider})=> divider && css`
      > div:last-child{
        display:none;
      }`
    }
  }
`;
export interface ListViewHeaderCustomProps extends KListViewHeaderProps{
$hasHeader?:boolean;
}
const StyledListViewHeader = styled(KListViewHeader)<ListViewHeaderCustomProps>`
  ${({$hasHeader})=>$hasHeader && css`
    padding: ${rem(5)} ${rem(10)};
  `}
`
const StyledListViewFooter = styled(KListViewFooter)``;
const MyListItem = ({as='div',divider,children,...rest})=>{
  if(as === "button"){
    return <Button{...rest} style={!divider?{marginBottom:"10px"}:{}}>{children}</Button>;
  }
  const StyledWrapper = styled[as]`
    padding: ${rem(5)} ${rem(10)};
    :hover{
      background:${colors.gray100};
    }
    ${({disabled})=> disabled && css`
      cursor:not-allowed;
      opacity: .8;
    `}
    ${
      as === "a" && css`
      cursor:pointer;
      text-decoration: none;
      color: ${colors.gray500};
      :hover{
        color: ${colors.purple};
      }
      &[disabled]{
        cursor:not-allowed;
        color: initial;
      }
    `}
    ${
      as==="span" && css`
      align-self:flex-start;
      `
    }
  `
  return <StyledWrapper {...rest}>{children}</StyledWrapper>
}
export function ListView({
  header:Header,
  footer:Footer,
  item:Item,
  divider=false,
  direction="vertical",
  onClick=()=>undefined,
  disabled=false,
  dataItemKey="label",
  as="a",
  ...props
}: ListViewProps) {
  const ListItem = useCallback<ComponentType<any>>(({dataItem,index,...rest})=>{
    function handleClick(e){
      if(disabled|| dataItem.disabled) {
        e.preventDefault();
        return;
      };
      onClick(dataItem,index, e);
    }
    const anchorProps = as==="a" && typeof dataItem!=="string"?{
      ...(dataItem.href ? { 
          href:dataItem?.href,
          target:dataItem["target"]
        }:{}
      )
    }:{};
    return(<>
      <MyListItem as={as} {...anchorProps} size={"small"} divider={divider} direction={direction} disabled={disabled||dataItem?.disabled} onClick={handleClick}>
        {
          Item
            ?<Item dataItem={dataItem}  {...rest}/>
            :typeof dataItem==="string"
              ?dataItem
                :dataItem?.[dataItemKey]
        }
      </MyListItem>
      {divider && <Divider type={direction==="vertical"?"horizontal":"vertical"}  thickness={1} />}
      </>
    )
  },[as, direction, disabled, Item, divider,dataItemKey, onClick])
  const MyHeader = props=><StyledListViewHeader $hasHeader={!!Header}>
    {Header &&<Header {...props}/>}
  </StyledListViewHeader>
  const MyFooter = props=><StyledListViewFooter>
    {Footer&& <Footer {...props}/>}
  </StyledListViewFooter>

  return (
    <StyledListView 
    header={MyHeader}
    footer={MyFooter}
    item={ListItem}
    direction={direction}
    divider={divider}
    {...props}
    />
  );
}

export default ListView;
