import { WindowProps, WindowActionsBar, DialogHandle } from '@progress/kendo-react-dialogs';
import { forwardRef, useEffect, useRef } from 'react';
import { StyledModal } from './styles';

/* eslint-disable-next-line */
export interface ModalProps extends WindowProps {
  /** The title of the modal. */
  title?: React.ReactNode | string;
  /** The content of the modal action bar. */
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  actionBarContent?: any;
  /** Set if the modal is draggable. Defaults to false. */
  draggable?: boolean;
  /** Set if the modal will change stage on title double click.. Defaults to false. */
  doubleClickStageChange?: boolean;
  /** Set if the modal is resizable. Defaults to false. */
  resizable?: boolean;
  /** Set an overlay backdrop behind the modal. Defaults to false. */
  backdrop?: boolean;
  /** Specifies if the Window will render the maximize button. */
  maximizeButton?: React.ComponentType<any>;
  /** Specifies if the Window will render the minimize button. */
  minimizeButton?: React.ComponentType<any>;
  /** Specifies the component children */
  children?: any;
  /** todo */
  variant?: 'primary' | 'secondary' | 'success' | 'warning' | 'error';
  /** Specifies the initial top of the Window by percentage. */
  percentageTop?: number;
  /** Specifies the initial width of the Window by percentage. */
  percentageWidth?: number;
  /** Specifies the initial height of the Window by percentage.  */
  percentageHeight?: number;
  /** Specifies the initial left of the Window by percentage.  */
  percentageLeft?: number;
  /** content padding */
  contentPadding?: number;
  /** predefined modal size 
   * size: 
   *  - small: width 25%
   *  - medium: width 60%
   *  - large: width 80%
  */
  size?: "small" | "medium" | "large";
  /**
   * show html title attribute on modal title hover when its trucated
   */
  showTitleOnEllipsis?: boolean;
}
const modalSize = { //default sizes for responsive modal
  small:{percentageWidth:25},
  medium:{percentageWidth:60},
  large:{percentageWidth:80},
}
export const Modal = forwardRef(({
  actionBarContent,
  draggable = false,
  doubleClickStageChange = false,
  maximizeButton = () => null,
  minimizeButton = () => null,
  resizable = false,
  backdrop = false,
  children,
  variant = 'primary',
  title,
  percentageWidth,
  percentageHeight,
  percentageLeft,
  percentageTop,
  contentPadding=20,
  size="medium",
  className="",
  showTitleOnEllipsis=true,
  ...props
}: ModalProps,modalRef) => {
  function getInitialSize() {
    const _percentageWidth = percentageWidth || modalSize[size]?.percentageWidth || modalSize["medium"].percentageWidth;
    const documentElement = document.documentElement ||{};
    let initialHeight,initialWidth,initialLeft,initialTop;
    const _modalSize: {
      initialWidth?:number,
      initialHeight?:number,
      initialTop?:number,
      initialLeft?:number
    } = {};
    try {
      const w = Math.max(
        documentElement.clientWidth || 0,
        window.innerWidth || 0
      );
      const h = Math.max(
        documentElement.clientHeight || 0,
        window.innerHeight || 0
      );
      initialWidth = Math.floor((w * _percentageWidth) / 100);
      _modalSize.initialWidth = initialWidth;
      if(percentageHeight){
        initialHeight = Math.floor((h * percentageHeight) / 100);
      _modalSize.initialHeight = initialHeight;
      }
      if(percentageTop){
        initialTop = Math.floor((h * percentageTop) / 100);
      _modalSize.initialTop = initialTop;
      }
      if(percentageLeft){
        initialLeft = Math.floor((w * percentageLeft) / 100);
      _modalSize.initialLeft = initialLeft;
      }
      return _modalSize;
    } catch (e) {
      return { };
    }
  }
  useEffect(()=>{
    showTitleOnEllipsis && addTitleOnEllipsis();
  },[showTitleOnEllipsis])
  const modalContainerRef = useRef<DialogHandle | null>(null);
  function addTitleOnEllipsis(){
    let modalContainerElem: any = document; 
    const modalContainer = modalContainerRef.current;
    if(modalContainer && modalContainer.element){
      modalContainerElem = modalContainer.element;
    }
    const titleElem = modalContainerElem.querySelector(".k-window-title");
    titleElem && titleElem.addEventListener("mouseenter",function(this:any,e:any){
      if(this.offsetWidth < this.scrollWidth && !this.getAttribute('title')){
        titleElem.setAttribute("title",e?.target?.textContent);
      }
    })
  }

  return (
    <StyledModal
      ref={(element) => {
        modalContainerRef.current = element;
        if (typeof modalRef === 'function') {
          modalRef(element);
        }else if(typeof modalRef === 'object' && modalRef !== null){
          modalRef.current = element;
        }
      }}
      draggable={draggable}
      maximizeButton={maximizeButton}
      minimizeButton={minimizeButton}
      resizable={resizable}
      modal={backdrop}
      className={actionBarContent ? `has-action-bar ${className}` : className}
      variant={variant}
      title={title}
      contentPadding={contentPadding}
      {...getInitialSize()}
      {...props}
    >
      {children}
      {actionBarContent && (
        // TODO: Refactor this so we don't need to pass the action bar as a prop
        <WindowActionsBar layout="end">{actionBarContent}</WindowActionsBar>
      )}
    </StyledModal>
  );
});

export default Modal;
